############
#Functions for implementing modified Cook's Distance
############
adaptiveold<-function(x,y,M,j,ran=FALSE) {
  
  #Function for combining delete j approach and confirmatory procedure using modified Cook's Distance
  #x: covariate data
  #y: response data
  #M: number of samples
  #j: size of deleted subsets
  #ran: if TRUE deleted subsets are uniform over 1 to j
  
  xcur<-x
  ycur<-y
  res<-rep(0,length(y))
  obs<-1:length(y)
  
  step1<-dcookold(x=xcur,y=ycur,M=M,j=j,ran=ran)
  ind1<-which(step1>1)
  
  if(length(ind1)>0 & length(ind1)<(length(y)-3)){
    step2<-confirmold(x=xcur,y=ycur,ind=ind1)
    ind2<-which(step2>1)
    if(length(ind2)>0) res[obs[ind1[ind2]]]<-1 
    
    return(res)
    
  }
  
  res[obs[ind1]]<-1
  
  return(res)
}
############
############
dcookold<-function(x,y,M,j,ran=FALSE) {
  
  #Uses modified Cook's Distance applied to data with subsets of size j deleted
  #x: covariate data
  #y: response data
  #M: number of samples
  #j: size of deleted subsets
  #ran: if TRUE deleted subsets are uniform over 1 to j
  
  n<-length(y)
  temp<-matrix(NA,M,n)
  
  s2<-(summary(lm(y~x))$sigma)^2
  xtx<-t(cbind(1,x))%*%cbind(1,x)
  p<-ncol(x)+1
  
  for(i in 1:M) {
    
    delete<-j
    if(ran==TRUE) delete<-round(runif(1,1,j))
    
    ind<-sample(x=1:n,size=delete,replace=FALSE)
    fit<-lm(y[-ind]~x[-ind,])
    change<-influence(fit)$coefficients
    res<-rep(0,n-delete)
    
    for(k in 1:(n-delete)) {
      res[k]<-change[k,]%*%xtx%*%change[k,]/(p*s2)
    }
    
    temp[i,-ind]<-res
    
  }
  
  return(apply(temp,2,max,na.rm=T))
}
############
############
confirmold<-function(x,y,ind) {
  
  #runs confirmatory procedure using modified Cook's Distance to check flagged observations
  #x: covariate data
  #y: response data
  #ind: observations flagged as influential
  
  xred<-as.matrix(x[-ind,])
  yred<-y[-ind]
  fitfull<-lm(yred~xred)
  coeffull<-fitfull$coef
  s2<-(summary(fitfull)$sigma)^2
  xtx<-t(cbind(1,xred))%*%cbind(1,xred)
  p<-ncol(xred)+1
  
  temp<-NULL
  
  for(i in ind) {
    
    fit<-lm(c(y[i],yred)~rbind(x[i,],xred))
    temp<-c(temp,(coeffull-fit$coef)%*%xtx%*%(coeffull-fit$coef)/(p*s2))
  }
  
  return(temp)
}
############
############
adaptivereal<-function(x,y,M,j,ran=FALSE) {
  
  #Function for combining delete j approach and confirmatory procedure using modified Cook's Distance
  #Returns values of modified Cook's distance
  #x: covariate data
  #y: response data
  #M: number of samples
  #j: size of deleted subsets
  #ran: if TRUE deleted subsets are uniform over 1 to j
  
  xcur<-x
  ycur<-y
  res<-rep(0,length(y))
  obs<-1:length(y)
  
  step1<-dcookold(x=xcur,y=ycur,M=M,j=j,ran=ran)
  ind1<-which(step1>1)
  
  if(length(ind1)>0 & length(ind1)<(length(y)-3)){
    step2<-confirmold(x=xcur,y=ycur,ind=ind1)
    ind2<-which(step2>1)
    if(length(ind2)>0) res[obs[ind1[ind2]]]<-1 
    
    temp<-rep(0,length(y))
    temp[ind1]<-step2
    
    return(list(res,cbind(step1,temp,cooks.distance(lm(y~x)))))
    
  }
  
  res[obs[ind1]]<-1
  
  return(list(res,cbind(step1,cooks.distance(lm(y~x)))))
}